@@ -1,21 +1,8 @@
-// File__Analyze -
-// Copyright (C) 2009-2012 MediaArea.net SARL, Info@MediaArea.net
-//
-// This library is free software: you can redistribute it and/or modify it
-// under the terms of the GNU Library General Public License as published by
-// the Free Software Foundation, either version 2 of the License, or
-// any later version.
-//
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public License
-// along with this library. If not, see <http://www.gnu.org/licenses/>.
-//
-//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+/*  Copyright (c) MediaArea.net SARL. All Rights Reserved.
+ *
+ *  Use of this source code is governed by a BSD-style license that can
+ *  be found in the License.html file in the root of the source tree.
+ */
 
 //---------------------------------------------------------------------------
 #ifndef MediaInfo_File__AnalyzeMinimizeSizeH
@@ -44,13 +31,16 @@ public :
     void    Open_Buffer_Init        (File__Analyze* Sub);
     void    Open_Buffer_Init        (File__Analyze* Sub, int64u File_Size);
     void    Open_Buffer_Continue    (                    const int8u* Buffer, size_t Buffer_Size);
-    void    Open_Buffer_Continue    (File__Analyze* Sub, const int8u* Buffer, size_t Buffer_Size);
+    void    Open_Buffer_Continue    (File__Analyze* Sub, const int8u* Buffer, size_t Buffer_Size, bool IsNewPacket=true, float64 Ratio=1.0);
     void    Open_Buffer_Continue    (File__Analyze* Sub, size_t Buffer_Size) {if (Element_Offset+Buffer_Size<=Element_Size) Open_Buffer_Continue(Sub, Buffer+Buffer_Offset+(size_t)Element_Offset, Buffer_Size); Element_Offset+=Buffer_Size;}
     void    Open_Buffer_Continue    (File__Analyze* Sub) {if (Element_Offset<=Element_Size) Open_Buffer_Continue(Sub, Buffer+Buffer_Offset+(size_t)Element_Offset, (size_t)(Element_Size-Element_Offset)); Element_Offset=Element_Size;}
     void    Open_Buffer_Position_Set(int64u File_Offset);
     #if MEDIAINFO_SEEK
     size_t  Open_Buffer_Seek        (size_t Method, int64u Value, int64u ID);
     #endif //MEDIAINFO_SEEK
+    #if MEDIAINFO_ADVANCED2
+    void    Open_Buffer_SegmentChange();
+    #endif //MEDIAINFO_ADVANCED2
     void    Open_Buffer_Unsynch     ();
     void    Open_Buffer_Update      ();
     void    Open_Buffer_Update      (File__Analyze* Sub);
@@ -68,20 +58,22 @@ public :
         int64u  StreamIDs[16];
         int8u   StreamIDs_Width[16];
         int8u   ParserIDs[16];
+        void    Event_Prepare (struct MediaInfo_Event_Generic* Event);
     #endif //MEDIAINFO_EVENTS
     #if MEDIAINFO_DEMUX
-        int8u   Demux_Level; //bit 0=frame, bit 1=container, bit 2=elementary (eg MPEG-TS), default with frame set
+        int8u   Demux_Level; //bit 0=frame, bit 1=container, bit 2=elementary (eg MPEG-TS), bit 3=ancillary (e.g. DTVCC), default with frame set
         bool    Demux_random_access;
         bool    Demux_UnpacketizeContainer;
         bool    Demux_IntermediateItemFound;
         size_t  Demux_Offset;
         int64u  Demux_TotalBytes;
+        File__Analyze* Demux_CurrentParser;
     #endif //MEDIAINFO_DEMUX
     Ztring  File_Name_WithoutDemux;
     bool   PTS_DTS_Needed;
     struct frame_info
     {
-        int64u Buffer_Offset;
+        int64u Buffer_Offset_End;
         int64u PCR; //In nanoseconds
         int64u PTS; //In nanoseconds
         int64u DTS; //In nanoseconds
@@ -89,7 +81,7 @@ public :
 
         frame_info()
         {
-            Buffer_Offset=0;
+            Buffer_Offset_End=(int64u)-1;
             PCR=(int64u)-1;
             PTS=(int64u)-1;
             DTS=(int64u)-1;
@@ -99,9 +91,36 @@ public :
     frame_info FrameInfo;
     frame_info FrameInfo_Previous;
     frame_info FrameInfo_Next;
+    std::vector<int64u> Offsets_Stream;
+    std::vector<int64u> Offsets_Buffer;
+    size_t              Offsets_Pos;
+    int8u*              OriginalBuffer;
+    size_t              OriginalBuffer_Size;
+    size_t              OriginalBuffer_Capacity;
+    #if defined(MEDIAINFO_EIA608_YES) || defined(MEDIAINFO_EIA708_YES)
+        struct servicedescriptor
+        {
+            string language;
+        };
+        typedef std::map<int8u, servicedescriptor> servicedescriptors608;
+        typedef std::map<int8u, servicedescriptor> servicedescriptors708;
+        struct servicedescriptors
+        {
+            #if defined(MEDIAINFO_EIA608_YES)
+                std::map<int8u, servicedescriptor> ServiceDescriptors608;
+            #endif
+            #if defined(MEDIAINFO_EIA708_YES)
+                std::map<int8u, servicedescriptor> ServiceDescriptors708;
+            #endif
+        };
+        servicedescriptors* ServiceDescriptors;
+    #endif
 
     //Out
     int64u PTS_Begin;                  //In nanoseconds
+    #if MEDIAINFO_ADVANCED2
+    int64u PTS_Begin_Segment;          //In nanoseconds
+    #endif //MEDIAINFO_ADVANCED2
     int64u PTS_End;                    //In nanoseconds
     int64u DTS_Begin;                  //In nanoseconds
     int64u DTS_End;                    //In nanoseconds
@@ -112,7 +131,9 @@ public :
     int64u Field_Count_Previous;
     int64u Field_Count_InThisBlock;
     int64u Frame_Count_NotParsedIncluded;
+    int64u FrameNumber_PresentationOrder;
     bool   Synched;                    //Data is synched
+    bool   UnSynched_IsNotJunk;        //Data is actually synched
     bool   MustExtendParsingDuration;  //Data has some substreams difficult to detect (e.g. captions), must wait a bit before final filling
 
 protected :
@@ -147,6 +168,9 @@ protected :
     virtual size_t Read_Buffer_Seek (size_t, int64u, int64u); //Temp, should be in File__Base caller
     size_t Read_Buffer_Seek_OneFramePerFile (size_t, int64u, int64u);
     #endif //MEDIAINFO_SEEK
+    #if MEDIAINFO_ADVANCED2
+    virtual void Read_Buffer_SegmentChange () {}; //Temp, should be in File__Base caller
+    #endif //MEDIAINFO_ADVANCED2
     virtual void Read_Buffer_Unsynched ()     {}; //Temp, should be in File__Base caller
     void Read_Buffer_Unsynched_OneFramePerFile ();
     virtual void Read_Buffer_Finalize ()      {}; //Temp, should be in File__Base caller
@@ -190,6 +214,7 @@ protected :
     //Header - Info
     void Header_Fill_Code (int64u Code);
     inline void Header_Fill_Code (int64u Code, const Ztring &) {Header_Fill_Code(Code);};
+    #define Header_Fill_Code2(A,B) Header_Fill_Code(A)
     void Header_Fill_Size (int64u Size);
 
     //***************************************************************************
@@ -300,6 +325,7 @@ public :
     void Get_B7_   (int64u  &Info);
     void Get_B8_   (int64u  &Info);
     void Get_B16_  (int128u &Info);
+    void Get_BF2_  (float32 &Info);
     void Get_BF4_  (float32 &Info);
     void Get_BF8_  (float64 &Info);
     void Get_BF10_ (float80 &Info);
@@ -313,6 +339,7 @@ public :
     #define Get_B7(Info, Name) Get_B7_(Info)
     #define Get_B8(Info, Name) Get_B8_(Info)
     #define Get_B16(Info, Name) Get_B16_(Info)
+    #define Get_BF2(Info, Name) Get_BF2_(Info)
     #define Get_BF4(Info, Name) Get_BF4_(Info)
     #define Get_BF8(Info, Name) Get_BF8_(Info)
     #define Get_BF10(Info, Name) Get_BF10_(Info)
@@ -508,9 +535,9 @@ public :
     // UUID
     //***************************************************************************
 
-    void Get_UUID (int128u &Info);
-    inline void Get_UUID (int128u &Info, const char*) {Get_UUID(Info);}
-    void Peek_UUID(int128u &Info);
+    inline void Get_UUID (int128u &Info) {Get_B16_(Info);}
+    inline void Get_UUID (int128u &Info, const char*) {Get_B16_(Info);}
+    inline void Peek_UUID(int128u &Info) {Peek_B16(Info);}
     inline void Skip_UUID(               const char*) {if (Element_Offset+16>Element_Size) {Trusted_IsNot(); return;} Element_Offset+=16;}
     #define Info_UUID(_INFO, _NAME) int128u _INFO; Get_UUID(_INFO, _NAME)
 
@@ -548,7 +575,7 @@ public :
     inline void Get_UE (int32u &Info, const char*) {Get_UE(Info);}
     inline void Get_SE (int32s &Info, const char*) {Get_SE(Info);}
     void Skip_UE();
-    void Skip_SE();
+    inline void Skip_SE() {Skip_UE();}
     inline void Skip_UE(              const char*) {Skip_UE();}
     inline void Skip_SE(              const char*) {Skip_SE();}
     #define Info_UE(_INFO, _NAME) int32u _INFO; Get_UE(_INFO, _NAME)
@@ -641,16 +668,20 @@ public :
     //***************************************************************************
 
     void Get_Local  (int64u Bytes, Ztring      &Info);
+    void Get_ISO_6937_2 (int64u Bytes, Ztring  &Info);
     void Get_ISO_8859_1 (int64u Bytes, Ztring  &Info);
     void Get_ISO_8859_2 (int64u Bytes, Ztring  &Info);
+    void Get_ISO_8859_5 (int64u Bytes, Ztring  &Info);
     void Get_String (int64u Bytes, std::string &Info);
     void Get_UTF8   (int64u Bytes, Ztring      &Info);
     void Get_UTF16  (int64u Bytes, Ztring      &Info);
     void Get_UTF16B (int64u Bytes, Ztring      &Info);
     void Get_UTF16L (int64u Bytes, Ztring      &Info);
     inline void Get_Local  (int64u Bytes, Ztring      &Info, const char*) {Get_Local(Bytes, Info);}
+    inline void Get_ISO_6937_2 (int64u Bytes, Ztring  &Info, const char*) {Get_ISO_6937_2(Bytes, Info);}
     inline void Get_ISO_8859_1 (int64u Bytes, Ztring  &Info, const char*) {Get_ISO_8859_1(Bytes, Info);}
     inline void Get_ISO_8859_2 (int64u Bytes, Ztring  &Info, const char*) {Get_ISO_8859_2(Bytes, Info);}
+    inline void Get_ISO_8859_5 (int64u Bytes, Ztring  &Info, const char*) {Get_ISO_8859_5(Bytes, Info);}
     inline void Get_String (int64u Bytes, std::string &Info, const char*) {Get_String(Bytes, Info);}
     inline void Get_UTF8   (int64u Bytes, Ztring      &Info, const char*) {Get_UTF8(Bytes, Info);}
     inline void Get_UTF16  (int64u Bytes, Ztring      &Info, const char*) {Get_UTF16(Bytes, Info);}
@@ -659,11 +690,13 @@ public :
     void Peek_Local (int64u Bytes, Ztring      &Info);
     void Peek_String(int64u Bytes, std::string &Info);
     void Skip_Local (int64u Bytes,                    const char*) {if (Element_Offset+Bytes>Element_Size) {Trusted_IsNot(); return;} Element_Offset+=(size_t)Bytes;}
+    void Skip_ISO_6937_2(int64u Bytes,                const char*) {if (Element_Offset+Bytes>Element_Size) {Trusted_IsNot(); return;} Element_Offset+=(size_t)Bytes;}
     void Skip_String(int64u Bytes,                    const char*) {if (Element_Offset+Bytes>Element_Size) {Trusted_IsNot(); return;} Element_Offset+=(size_t)Bytes;}
     void Skip_UTF8  (int64u Bytes,                    const char*) {if (Element_Offset+Bytes>Element_Size) {Trusted_IsNot(); return;} Element_Offset+=(size_t)Bytes;}
     void Skip_UTF16B(int64u Bytes,                    const char*) {if (Element_Offset+Bytes>Element_Size) {Trusted_IsNot(); return;} Element_Offset+=(size_t)Bytes;}
     void Skip_UTF16L(int64u Bytes,                    const char*) {if (Element_Offset+Bytes>Element_Size) {Trusted_IsNot(); return;} Element_Offset+=(size_t)Bytes;}
     #define Info_Local(_BYTES, _INFO, _NAME)  Ztring _INFO; Get_Local (_BYTES, _INFO, _NAME)
+    #define Info_ISO_6937_2(_BYTES, _INFO, _NAME)  Ztring _INFO; Info_ISO_6937_2(_BYTES, _INFO, _NAME)
     #define Info_UTF8(_BYTES, _INFO, _NAME)   Ztring _INFO; Get_UTF8  (_BYTES, _INFO, _NAME)
     #define Info_UTF16B(_BYTES, _INFO, _NAME) Ztring _INFO; Get_UTF16B(_BYTES, _INFO, _NAME)
     #define Info_UTF16L(_BYTES, _INFO, _NAME) Ztring _INFO; Get_UTF16L(_BYTES, _INFO, _NAME)
@@ -828,62 +861,38 @@ public :
     bool Get_TB_ ()                                              {bool Temp; Get_TB_(Temp); return Temp;}
     void Get_T1_ (int8u  Bits, int8u   &Info);
     void Get_T2_ (int8u  Bits, int16u  &Info);
-    void Get_T3_ (int8u  Bits, int32u  &Info);
     void Get_T4_ (int8u  Bits, int32u  &Info);
-    void Get_T5_ (int8u  Bits, int64u  &Info);
-    void Get_T6_ (int8u  Bits, int64u  &Info);
-    void Get_T7_ (int8u  Bits, int64u  &Info);
     void Get_T8_ (int8u  Bits, int64u  &Info);
     #define Get_BT(Bits, Info, Name) Get_BT_(Bits, Info)
     #define Get_TB(      Info, Name) Get_TB_(      Info)
     #define Get_T1(Bits, Info, Name) Get_T1_(Bits, Info)
     #define Get_T2(Bits, Info, Name) Get_T2_(Bits, Info)
-    #define Get_T3(Bits, Info, Name) Get_T3_(Bits, Info)
     #define Get_T4(Bits, Info, Name) Get_T4_(Bits, Info)
-    #define Get_T5(Bits, Info, Name) Get_T5_(Bits, Info)
-    #define Get_T6(Bits, Info, Name) Get_T6_(Bits, Info)
-    #define Get_T7(Bits, Info, Name) Get_T7_(Bits, Info)
     #define Get_T8(Bits, Info, Name) Get_T8_(Bits, Info)
     void Peek_BT(int8u  Bits, int32u  &Info);
     void Peek_TB(              bool    &Info);
     bool Peek_TB()                                              {bool Temp; Peek_TB(Temp); return Temp;}
     void Peek_T1(int8u  Bits, int8u   &Info);
     void Peek_T2(int8u  Bits, int16u  &Info);
-    void Peek_T3(int8u  Bits, int32u  &Info);
     void Peek_T4(int8u  Bits, int32u  &Info);
-    void Peek_T5(int8u  Bits, int64u  &Info);
-    void Peek_T6(int8u  Bits, int64u  &Info);
-    void Peek_T7(int8u  Bits, int64u  &Info);
     void Peek_T8(int8u  Bits, int64u  &Info);
     inline void Skip_BT_(size_t Bits) {BT->Skip(Bits);}
     inline void Skip_TB_(           ) {BT->SkipB();}
     inline void Skip_T1_(int8u  Bits) {BT->Skip1(Bits);}
     inline void Skip_T2_(int8u  Bits) {BT->Skip2(Bits);}
-    inline void Skip_T3_(int8u  Bits) {BT->Skip4(Bits);}
     inline void Skip_T4_(int8u  Bits) {BT->Skip4(Bits);}
-    inline void Skip_T5_(int8u  Bits) {BT->Skip8(Bits);}
-    inline void Skip_T6_(int8u  Bits) {BT->Skip8(Bits);}
-    inline void Skip_T7_(int8u  Bits) {BT->Skip8(Bits);}
     inline void Skip_T8_(int8u  Bits) {BT->Skip8(Bits);}
     #define Skip_BT(Bits, Name) Skip_BT_(Bits)
     #define Skip_TB(      Name) Skip_TB_()
     #define Skip_T1(Bits, Name) Skip_T1_(Bits)
     #define Skip_T2(Bits, Name) Skip_T2_(Bits)
-    #define Skip_T3(Bits, Name) Skip_T3_(Bits)
     #define Skip_T4(Bits, Name) Skip_T4_(Bits)
-    #define Skip_T5(Bits, Name) Skip_T5_(Bits)
-    #define Skip_T6(Bits, Name) Skip_T6_(Bits)
-    #define Skip_T7(Bits, Name) Skip_T7_(Bits)
     #define Skip_T8(Bits, Name) Skip_T8_(Bits)
     #define Info_BT(_BITS, _INFO, _NAME) Skip_BT_(_BITS)
     #define Info_TB(_INFO, _NAME)        Skip_TB_(     )
     #define Info_T1(_BITS, _INFO, _NAME) Skip_T1_(_BITS)
     #define Info_T2(_BITS, _INFO, _NAME) Skip_T2_(_BITS)
-    #define Info_T3(_BITS, _INFO, _NAME) Skip_T3_(_BITS)
     #define Info_T4(_BITS, _INFO, _NAME) Skip_T4_(_BITS)
-    #define Info_T5(_BITS, _INFO, _NAME) Skip_T5_(_BITS)
-    #define Info_T6(_BITS, _INFO, _NAME) Skip_T6_(_BITS)
-    #define Info_T7(_BITS, _INFO, _NAME) Skip_T7_(_BITS)
     #define Info_T8(_BITS, _INFO, _NAME) Skip_T8_(_BITS)
 
     #define TEST_TB_GET(_CODE, _NAME) \
@@ -999,7 +1008,7 @@ public :
     #endif //SIZE_T_IS_LONG
     ZtringListList Fill_Temp;
     void Fill_Flush ();
-    size_t Fill_Parameter(stream_t StreamKind, generic StreamPos);
+    static size_t Fill_Parameter(stream_t StreamKind, generic StreamPos);
 
     const Ztring &Retrieve_Const (stream_t StreamKind, size_t StreamPos, size_t Parameter, info_t KindOfInfo=Info_Text);
     Ztring Retrieve (stream_t StreamKind, size_t StreamPos, size_t Parameter, info_t KindOfInfo=Info_Text);
@@ -1056,27 +1065,6 @@ public :
     bool Element_IsNotFinished ();
     bool Element_IsWaitingForMoreData ();
 
-    //Begin
-    #define FILLING_BEGIN() \
-        if (Element_IsOK()) \
-        {
-
-    #define FILLING_BEGIN_PRECISE() \
-        if (Element_Offset!=Element_Size) \
-            Trusted_IsNot("Size error"); \
-        else if (Element_IsOK()) \
-        {
-
-    //Else
-    #define FILLING_ELSE() \
-        } \
-        else \
-        { \
-
-    //End
-    #define FILLING_END() \
-        }
-
     //***************************************************************************
     // Merging
     //***************************************************************************
@@ -1088,6 +1076,8 @@ public :
     size_t Merge(File__Analyze &ToAdd, bool Erase=true); //Merge 2 File_Base
     size_t Merge(File__Analyze &ToAdd, stream_t StreamKind, size_t StreamPos_From, size_t StreamPos_To, bool Erase=true); //Merge 2 streams
 
+    void CodecID_Fill           (const Ztring &Value, stream_t StreamKind, size_t StreamPos, infocodecid_format_t Format, stream_t StreamKind_CodecID=Stream_Max);
+
     //***************************************************************************
     // Finalize
     //***************************************************************************
@@ -1102,7 +1092,7 @@ protected :
     void Streams_Finish_StreamOnly_Video(size_t StreamPos);
     void Streams_Finish_StreamOnly_Audio(size_t StreamPos);
     void Streams_Finish_StreamOnly_Text(size_t StreamPos);
-    void Streams_Finish_StreamOnly_Chapters(size_t StreamPos);
+    void Streams_Finish_StreamOnly_Other(size_t StreamPos);
     void Streams_Finish_StreamOnly_Image(size_t StreamPos);
     void Streams_Finish_StreamOnly_Menu(size_t StreamPos);
     void Streams_Finish_InterStreams();
@@ -1115,6 +1105,8 @@ protected :
     void Streams_Finish_Cosmetic_Chapters(size_t StreamPos);
     void Streams_Finish_Cosmetic_Image(size_t StreamPos);
     void Streams_Finish_Cosmetic_Menu(size_t StreamPos);
+    void Streams_Finish_HumanReadable();
+    void Streams_Finish_HumanReadable_PerStream(stream_t StreamKind, size_t StreamPos, size_t Parameter);
 
     void Tags ();
     void Video_FrameRate_Rounding (size_t Pos, video Parameter);
@@ -1127,7 +1119,6 @@ protected :
     void Kilo_Kilo123           (stream_t StreamKind, size_t StreamPos, size_t Parameter);
     void Value_Value123         (stream_t StreamKind, size_t StreamPos, size_t Parameter);
     void YesNo_YesNo            (stream_t StreamKind, size_t StreamPos, size_t Parameter);
-    void CodecID_Fill           (const Ztring &Value, stream_t StreamKind, size_t StreamPos, infocodecid_format_t Format);
 
     //***************************************************************************
     //
@@ -1164,15 +1155,17 @@ private :
     //***************************************************************************
 
     void Buffer_Clear(); //Clear the buffer
-    bool Open_Buffer_Continue_Loop();
 protected :
     //Buffer
+    bool Open_Buffer_Continue_Loop();
     const int8u* Buffer;
 public : //TO CHANGE
     size_t Buffer_Size;
     int64u Buffer_TotalBytes;
     int64u Buffer_TotalBytes_FirstSynched;
+    int64u Buffer_TotalBytes_LastSynched;
     int64u Buffer_PaddingBytes;
+    int64u Buffer_JunkBytes;
     float64 Stream_BitRateFromContainer;
 protected :
     int8u* Buffer_Temp;
@@ -1194,9 +1187,7 @@ protected :
     bool FileHeader_Begin_XML(tinyxml2::XMLDocument &Document);
     bool Synchronize_0x000001();
 public:
-    static void Streams_Accept_TestContinuousFileNames_Static(ZtringList &File_Names, bool IsReferenced);
-protected:
-    void Streams_Accept_TestContinuousFileNames();
+    void TestContinuousFileNames(size_t CountOfFiles=24, Ztring FileExtension=Ztring());
 
 private :
 
@@ -1293,7 +1284,7 @@ public :
         ContentType_Synchro
     };
     #if MEDIAINFO_DEMUX
-        void Demux (const int8u* Buffer, size_t Buffer_Size, contenttype ContentType);
+        void Demux (const int8u* Buffer, size_t Buffer_Size, contenttype ContentType, const int8u* OriginalBuffer=NULL, size_t OriginalBuffer_Size=0);
         virtual bool Demux_UnpacketizeContainer_Test() {return true;}
         bool Demux_UnpacketizeContainer_Test_OneFramePerFile();
         void Demux_UnpacketizeContainer_Demux(bool random_access=true);
@@ -1309,6 +1300,20 @@ public :
 
     int64u  Unsynch_Frame_Count;
 
+    //AES
+    #if MEDIAINFO_AES
+        AESdecrypt* AES;
+        int8u*      AES_IV;
+        int8u*      AES_Decrypted;
+        size_t      AES_Decrypted_Size;
+    #endif //MEDIAINFO_AES
+
+    //MD5
+    #if MEDIAINFO_MD5
+        struct MD5Context*  MD5;
+        int64u              Md5_ParseUpTo;
+    #endif //MEDIAINFO_MD5
+
     #if MEDIAINFO_SEEK
     private:
         bool Seek_Duration_Detected;
@@ -1319,7 +1324,8 @@ public :
         bool    Config_Ibi_Create;
         int64u  Ibi_SynchronizationOffset_Current;
         int64u  Ibi_SynchronizationOffset_BeginOfFrame;
-        ibi::stream* IbiStream;
+        ibi     Ibi; //If Main only
+        ibi::stream* IbiStream; //If sub only
         size_t  Ibi_Read_Buffer_Seek        (size_t Method, int64u Value, int64u ID);
         void    Ibi_Read_Buffer_Unsynched   ();
         void    Ibi_Stream_Finish           ();
