@@ -1,21 +1,8 @@
-// File_Flv - Info for Flash files
-// Copyright (C) 2005-2012 MediaArea.net SARL, Info@MediaArea.net
-//
-// This library is free software: you can redistribute it and/or modify it
-// under the terms of the GNU Library General Public License as published by
-// the Free Software Foundation, either version 2 of the License, or
-// any later version.
-//
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public License
-// along with this library. If not, see <http://www.gnu.org/licenses/>.
-//
-//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+/*  Copyright (c) MediaArea.net SARL. All Rights Reserved.
+ *
+ *  Use of this source code is governed by a BSD-style license that can
+ *  be found in the License.html file in the root of the source tree.
+ */
 
 //---------------------------------------------------------------------------
 //
@@ -48,6 +35,9 @@
 #if defined(MEDIAINFO_AVC_YES)
     #include "MediaInfo/Video/File_Avc.h"
 #endif
+#if defined(MEDIAINFO_HEVC_YES)
+    #include "MediaInfo/Video/File_Hevc.h"
+#endif
 #if defined(MEDIAINFO_AAC_YES)
     #include "MediaInfo/Audio/File_Aac.h"
 #endif
@@ -62,6 +52,9 @@
     #include "MediaInfo/MediaInfo_Events.h"
 #endif //MEDIAINFO_EVENTS
 #include <algorithm>
+#if MEDIAINFO_DEMUX
+    #include "base64.h"
+#endif //MEDIAINFO_DEMUX
 //---------------------------------------------------------------------------
 
 namespace MediaInfoLib
@@ -89,7 +82,7 @@ const int16u Flv_SamplingRate[]=
     8000, //Special case for Nellymoser 8kHz mono
 };
 
-const char* Flv_Format_Audio[]=
+const char* Flv_Format_Audio[16]=
 {
     "PCM",
     "ADPCM",
@@ -109,7 +102,7 @@ const char* Flv_Format_Audio[]=
     "",
 };
 
-const char* Flv_Format_Profile_Audio[]=
+const char* Flv_Format_Profile_Audio[16]=
 {
     "",
     "",
@@ -129,7 +122,7 @@ const char* Flv_Format_Profile_Audio[]=
     "",
 };
 
-const char* Flv_Codec_Audio[]=
+const char* Flv_Codec_Audio[16]=
 {
     "Uncompressed",
     "ADPCM",
@@ -149,7 +142,7 @@ const char* Flv_Codec_Audio[]=
     "",
 };
 
-const char* Flv_CodecID_Hint_Audio[]=
+const char* Flv_CodecID_Hint_Audio[16]=
 {
     "",
     "",
@@ -169,7 +162,7 @@ const char* Flv_CodecID_Hint_Audio[]=
     "",
 };
 
-const char* Flv_Format_Video[]=
+const char* Flv_Format_Video[16]=
 {
     "",
     "",
@@ -183,13 +176,13 @@ const char* Flv_Format_Video[]=
     "",
     "",
     "",
-    "",
+    "HEVC",
     "",
     "",
     "",
 };
 
-const char* Flv_Format_Profile_Video[]=
+const char* Flv_Format_Profile_Video[16]=
 {
     "",
     "",
@@ -209,7 +202,7 @@ const char* Flv_Format_Profile_Video[]=
     "",
 };
 
-const char* Flv_Codec_Video[]=
+const char* Flv_Codec_Video[16]=
 {
     "",
     "",
@@ -223,13 +216,13 @@ const char* Flv_Codec_Video[]=
     "",
     "",
     "",
-    "",
+    "HEVC",
     "",
     "",
     "",
 };
 
-const char* Flv_CodecID_Hint_Video[]=
+const char* Flv_CodecID_Hint_Video[16]=
 {
     "",
     "",
@@ -597,15 +590,33 @@ void File_Flv::Streams_Finish_PerStream(stream_t StreamKind)
 // Buffer - File header
 //***************************************************************************
 
+//---------------------------------------------------------------------------
+bool File_Flv::FileHeader_Begin()
+{
+    //Synchro
+    if (3>Buffer_Size)
+        return false;
+    if (Buffer[0]!=0x46 //"FLV"
+     || Buffer[1]!=0x4C
+     || Buffer[2]!=0x56)
+    {
+        Reject();
+        return false;
+    }
+    if (9>Buffer_Size)
+        return false;
+
+    return true;
+}
+
 //---------------------------------------------------------------------------
 void File_Flv::FileHeader_Parse()
 {
     //Parsing
     Element_Begin1("FLV header");
-    std::string Signature;
     int32u Size;
     int8u  Version, Flags;
-    Get_String(3, Signature,                                    "Signature");
+    Skip_String(3,                                              "Signature");
     Get_B1 (Version,                                            "Version");
     Get_B1 (Flags,                                              "Flags");
         Get_Flags (Flags, 0, video_stream_Count,                "Video");
@@ -617,7 +628,7 @@ void File_Flv::FileHeader_Parse()
 
     FILLING_BEGIN();
         //Integrity
-        if (Signature!="FLV" || Version==0 || Size<9)
+        if (Version==0 || Size<9)
         {
             Reject();
             return;
@@ -636,18 +647,30 @@ void File_Flv::FileHeader_Parse()
         if (video_stream_Count)
         {
             Stream_Prepare(Stream_Video);
+            #if MEDIAINFO_DEMUX
+                if (Config->Demux_ForceIds_Get())
+                    Fill(Stream_Video, 0, Video_ID, 9);
+            #endif //MEDIAINFO_DEMUX
             video_stream_FrameRate_Detected=false;
         }
         else
             video_stream_FrameRate_Detected=true;
         if (audio_stream_Count)
+        {
             Stream_Prepare(Stream_Audio);
+            #if MEDIAINFO_DEMUX
+                if (Config->Demux_ForceIds_Get())
+                    Fill(Stream_Audio, 0, Audio_ID, 8);
+            #endif //MEDIAINFO_DEMUX
+        }
 
         if (Version>1)
         {
             Finish();
             return; //Version more than 1 is not supported
         }
+    FILLING_ELSE()
+        Reject();
     FILLING_END();
 }
 
@@ -658,21 +681,33 @@ void File_Flv::FileHeader_Parse()
 //---------------------------------------------------------------------------
 bool File_Flv::Synchronize()
 {
+    if (File_Offset+Buffer_Offset+4==File_Size)
+        return true; // Used by seek from end
+
     //Synchronizing
     while (Buffer_Offset+15<=Buffer_Size)
     {
-        if (Buffer[Buffer_Offset+4]==0x08 || Buffer[Buffer_Offset+4]==0x09)
+        int32u BodyLength=BigEndian2int24u(Buffer+Buffer_Offset+5);
+        if ((Buffer[Buffer_Offset  ]
+            || Buffer[Buffer_Offset+1]
+            || Buffer[Buffer_Offset+2]
+            || Buffer[Buffer_Offset+3]>=11)
+            && File_Offset+Buffer_Offset+15+BodyLength==File_Size)
+            break; //Last block
+        if (File_Offset+Buffer_Offset+15+BodyLength<File_Size)
         {
-            int32u BodyLength=BigEndian2int24u(Buffer+Buffer_Offset+5);
-            if (File_Offset+Buffer_Offset+15+BodyLength==File_Size)
-                break; //Last block
-            if (File_Offset+Buffer_Offset+15+BodyLength<File_Size)
-            {
-                if (Buffer_Offset+15+BodyLength+15>Buffer_Size)
-                    return false; //Need more data
+            if (Buffer_Offset+15+BodyLength+15>Buffer_Size)
+                return false; //Need more data
 
-                if (Buffer[Buffer_Offset+15+BodyLength+4]==0x08 || Buffer[Buffer_Offset+15+BodyLength+4]==0x09)
-                    break;
+            if ((Buffer[Buffer_Offset  ]
+              || Buffer[Buffer_Offset+1]
+              || Buffer[Buffer_Offset+2]
+              || Buffer[Buffer_Offset+3]>=11)
+             && (BigEndian2int32u(Buffer+Buffer_Offset+15+BodyLength)==11+BodyLength // PreviousTagSize
+              || BigEndian2int32u(Buffer+Buffer_Offset+15+BodyLength)==BodyLength)) // PreviousTagSize without 11, found in some buggy files
+            {
+                 PreviousTagSize_Add11=(BigEndian2int32u(Buffer+Buffer_Offset+15+BodyLength)==BodyLength)?0:11;
+                 break;
             }
         }
 
@@ -687,6 +722,31 @@ bool File_Flv::Synchronize()
     return true;
 }
 
+//---------------------------------------------------------------------------
+bool File_Flv::Synched_Test()
+{
+    if (File_Offset+Buffer_Offset+4==File_Size)
+        return true; // Used by seek from end
+
+    //Must have enough buffer for having header
+    if (Buffer_Offset+15>Buffer_Size)
+        return false;
+
+    //Quick test of synchro
+    if (Buffer[Buffer_Offset  ]==0
+     && Buffer[Buffer_Offset+1]==0
+     && Buffer[Buffer_Offset+2]==0
+     && Buffer[Buffer_Offset+3]<PreviousTagSize_Add11
+     && File_Offset+Buffer_Offset>9)
+    {
+        Synched=false;
+        return true;
+    }
+
+    //We continue
+    return true;
+}
+
 //***************************************************************************
 // Buffer - Global
 //***************************************************************************
@@ -717,7 +777,7 @@ void File_Flv::Header_Parse()
         Get_B4 (PreviousTagSize,                                "PreviousTagSize");
 
         //Filling
-        Header_Fill_Code((int64u)-1, "End");
+        Header_Fill_Code((int64u)-1, "End Of File");
         Header_Fill_Size(4);
         return;
     }
@@ -759,6 +819,9 @@ void File_Flv::Header_Parse()
             if (!Searching_Duration || Stream[StreamKind].TimeStamp==(int32u)-1)
                 Stream[StreamKind].TimeStamp=Time;
         }
+
+        if (Type==0)
+            Trusted_IsNot("Wrong type");
     }
     else
     {
@@ -782,14 +845,11 @@ void File_Flv::Data_Parse()
         case 0x12 : meta(); break;
         case 0xFA : Rm(); break;
         case (int64u)-1 :   //When searching the last frame
-                            if (PreviousTagSize>File_Size)
+                            if (8+PreviousTagSize>File_Size)
                             {
-                                //There is a problem, trying to sync
                                 Searching_Duration=false;
-                                MustSynchronize=true;
-                                Buffer_TotalBytes_FirstSynched_Max=File_Size;
-                                GoToFromEnd(65536);
-                                return;
+                                Open_Buffer_Unsynch(); //There is a problem, trying to sync
+                                PreviousTagSize=65536;
                             }
                             GoTo(File_Size-PreviousTagSize-8, "FLV");
                             return;
@@ -801,7 +861,7 @@ void File_Flv::Data_Parse()
 
     }
 
-    if (Searching_Duration && !MustSynchronize)
+    if (Searching_Duration)
     {
         if ((((Count_Get(Stream_Video)==0 || Stream[Stream_Video].TimeStamp!=(int32u)-1)
            && (Count_Get(Stream_Audio)==0 || Stream[Stream_Audio].TimeStamp!=(int32u)-1))
@@ -812,8 +872,7 @@ void File_Flv::Data_Parse()
         {
             //Trying to sync
             Searching_Duration=false;
-            MustSynchronize=true;
-            Buffer_TotalBytes_FirstSynched_Max=File_Size;
+            Open_Buffer_Unsynch(); //There is a problem, trying to sync
             GoToFromEnd(Header_Size+Element_Size+65536);
             return;
         }
@@ -857,8 +916,6 @@ void File_Flv::video()
         return;
     }
 
-    Demux(Buffer+Buffer_Offset+(size_t)Element_Offset, (size_t)(Element_Size-Element_Offset), ContentType_MainStream);
-
     //Needed?
     if (!video_stream_Count && Config->ParseSpeed<1)
         return; //No more need of Video stream
@@ -878,15 +935,14 @@ void File_Flv::video()
         {
             if (Count_Get(Stream_Video)==0)
                 Stream_Prepare(Stream_Video);
-            if (Codec<8)
-            {
-                Fill(Stream_Video, 0, Video_Format, Flv_Format_Video[Codec]);
-                Fill(Stream_Video, 0, Video_Format_Profile, Flv_Format_Profile_Video[Codec]);
-                Fill(Stream_Video, 0, Video_Codec, Flv_Codec_Video[Codec]);
-                Fill(Stream_Video, 0, Video_CodecID, Codec);
-                Fill(Stream_Video, 0, Video_CodecID_Hint, Flv_CodecID_Hint_Video[Codec]);
-                Fill(Stream_Video, 0, Video_BitDepth, 8); //FLV is not known to support another bit depth
-            }
+            Fill(Stream_Video, 0, Video_Format, Flv_Format_Video[Codec]);
+            Fill(Stream_Video, 0, Video_Format_Profile, Flv_Format_Profile_Video[Codec]);
+            Fill(Stream_Video, 0, Video_Codec, Flv_Codec_Video[Codec]);
+            Fill(Stream_Video, 0, Video_CodecID, Codec);
+            Fill(Stream_Video, 0, Video_CodecID_Hint, Flv_CodecID_Hint_Video[Codec]);
+            Fill(Stream_Video, 0, Video_BitDepth, 8); //FLV is not known to support another bit depth
+
+            MustSynchronize=true; // Now, synchronization test is possible
         }
 
         //Parsing video data
@@ -898,10 +954,19 @@ void File_Flv::video()
             case  5 : video_VP6(true); break;
             case  6 : video_ScreenVideo(2); break;
             case  7 : video_AVC(); break;
+            case 12 : video_HEVC(); break;
             default : Skip_XX(Element_Size-Element_Offset,      "Unknown");
                       video_stream_Count=false; //No more need of Video stream;
         }
     FILLING_END();
+
+    #if MEDIAINFO_DEMUX
+        int8u Demux_Level_old=Demux_Level;
+        if (Stream[Stream_Video].Parser && Stream[Stream_Video].Parser->Demux_Level==2)
+            Demux_Level=4;
+        Demux(Buffer+Buffer_Offset+1, (size_t)(Element_Size-1), ContentType_MainStream);
+        Demux_Level=Demux_Level_old;
+    #endif //MEDIAINFO_DEMUX
 }
 
 //---------------------------------------------------------------------------
@@ -1077,6 +1142,87 @@ void File_Flv::video_AVC()
                  video_stream_Count=false; //Unable to parse it
     }
 }
+
+//---------------------------------------------------------------------------
+void File_Flv::video_HEVC()
+{
+    int8u AVCPacketType;
+    Get_B1 (AVCPacketType,                                      "AVCPacketType"); Param_Info1(Flv_AVCPacketType(AVCPacketType));
+    Info_B3(CompositionTime,                                    "CompositionTime"); Param_Info1(Ztring::ToZtring((int32s)(CompositionTime+0xFF000000)));
+
+    switch (AVCPacketType)
+    {
+        case 0 :
+                #ifdef MEDIAINFO_HEVC_YES
+                    if (Stream[Stream_Video].Parser==NULL)
+                    {
+                        Stream[Stream_Video].Parser=new File_Hevc;
+                        Open_Buffer_Init(Stream[Stream_Video].Parser);
+                        ((File_Hevc*)Stream[Stream_Video].Parser)->MustParse_VPS_SPS_PPS=true;
+                        ((File_Hevc*)Stream[Stream_Video].Parser)->MustParse_VPS_SPS_PPS_FromFlv=true;
+                        ((File_Hevc*)Stream[Stream_Video].Parser)->MustSynchronize=false;
+                        ((File_Hevc*)Stream[Stream_Video].Parser)->SizedBlocks=true;
+                        #if MEDIAINFO_DEMUX
+                            if (Config->Demux_Hevc_Transcode_Iso14496_15_to_AnnexB_Get())
+                            {
+                                Stream[Stream_Video].Parser->Demux_Level=2; //Container
+                                Stream[Stream_Video].Parser->Demux_UnpacketizeContainer=true;
+                            }
+                        #endif //MEDIAINFO_DEMUX
+                    }
+
+                    //Parsing
+                    Open_Buffer_Continue(Stream[Stream_Video].Parser);
+
+                    //Demux
+                    #if MEDIAINFO_DEMUX
+                        switch (Config->Demux_InitData_Get())
+                        {
+                            case 0 :    //In demux event
+                                        Demux_Level=2; //Container
+                                        Demux(Buffer+Buffer_Offset+2, (size_t)(Element_Size-2), ContentType_Header);
+                                        break;
+                            case 1 :    //In field
+                                        {
+                                        std::string Data_Raw((const char*)(Buffer+Buffer_Offset+2), (size_t)(Element_Size-2));
+                                        std::string Data_Base64(Base64::encode(Data_Raw));
+                                        Fill(Stream_Video, StreamPos_Last, "Demux_InitBytes", Data_Base64);
+                                        (*Stream_More)[Stream_Video][StreamPos_Last](Ztring().From_Local("Demux_InitBytes"), Info_Options)=__T("N NT");
+                                        }
+                                        break;
+                            default :   ;
+                        }
+                    #endif //MEDIAINFO_DEMUX
+                #else
+                    Skip_XX(Element_Size-Element_Offset,        "HEVC Data");
+                    video_stream_Count=false; //Unable to parse it
+                #endif
+                break;
+        case 1 :
+                #ifdef MEDIAINFO_HEVC_YES
+                    if (Stream[Stream_Video].Parser==NULL)
+                    {
+                        //Data before header, this is wrong
+                        video_stream_Count=false;
+                        break;
+                    }
+
+                    //Parsing
+                    Open_Buffer_Continue(Stream[Stream_Video].Parser);
+
+                    //Disabling this stream
+                    if (Stream[Stream_Video].Parser->File_GoTo!=(int64u)-1 || Stream[Stream_Video].Parser->Count_Get(Stream_Video)>0 || (Config->ParseSpeed<1.0 && Stream[Stream_Video].PacketCount>=300))
+                         video_stream_Count=false;
+                #else
+                    Skip_XX(Element_Size-Element_Offset,        "HEVC Data");
+                    video_stream_Count=false; //Unable to parse it
+                #endif
+                break;
+        default: Skip_XX(Element_Size-Element_Offset,           "Unknown");
+                 video_stream_Count=false; //Unable to parse it
+    }
+}
+
 //---------------------------------------------------------------------------
 void File_Flv::audio()
 {
@@ -1090,8 +1236,6 @@ void File_Flv::audio()
         return;
     }
 
-    Demux(Buffer+Buffer_Offset+(size_t)Element_Offset+1, (size_t)(Element_Size-Element_Offset-1), ContentType_MainStream);
-
     //Needed?
     if (!audio_stream_Count && Config->ParseSpeed<1)
         return; //No more need of Audio stream
@@ -1115,6 +1259,11 @@ void File_Flv::audio()
         is_stereo=false; //Mono forced
     }
 
+    if (codec!=10) // AAC has an header
+    {
+        Demux(Buffer+Buffer_Offset+(size_t)(Element_Offset+1), (size_t)(Element_Size-Element_Offset-1), ContentType_MainStream);
+    }
+
     FILLING_BEGIN();
         if (Retrieve(Stream_Audio, 0, Audio_Format).empty())
         {
@@ -1140,6 +1289,8 @@ void File_Flv::audio()
                 Fill(Stream_Audio, 0, Audio_Codec_Settings_Firm, "SWF");
 
             }
+
+            MustSynchronize=true; // Now, synchronization test is possible
         }
 
         //Parsing audio data
@@ -1183,7 +1334,7 @@ void File_Flv::audio_AAC()
     switch (AACPacketType)
     {
         case 0 :
-                #if defined(MEDIAINFO_MPEG4_YES)
+                #if defined(MEDIAINFO_AAC_YES)
                     if (Stream[Stream_Audio].Parser==NULL)
                     {
                         Stream[Stream_Audio].Parser=new File_Aac;
@@ -1193,6 +1344,26 @@ void File_Flv::audio_AAC()
 
                     //Parsing
                     Open_Buffer_Continue(Stream[Stream_Audio].Parser);
+
+                    //Demux
+                    #if MEDIAINFO_DEMUX
+                        switch (Config->Demux_InitData_Get())
+                        {
+                            case 0 :    //In demux event
+                                        Demux_Level=2; //Container
+                                        Demux(Buffer+Buffer_Offset+2, (size_t)(Element_Size-2), ContentType_Header);
+                                        break;
+                            case 1 :    //In field
+                                        {
+                                        std::string Data_Raw((const char*)(Buffer+Buffer_Offset+2), (size_t)(Element_Size-2));
+                                        std::string Data_Base64(Base64::encode(Data_Raw));
+                                        Fill(Stream_Audio, StreamPos_Last, "Demux_InitBytes", Data_Base64);
+                                        (*Stream_More)[Stream_Audio][StreamPos_Last](Ztring().From_Local("Demux_InitBytes"), Info_Options)=__T("N NT");
+                                        }
+                                        break;
+                            default :   ;
+                        }
+                    #endif //MEDIAINFO_DEMUX
                 #else
                     Skip_XX(Element_Size-Element_Offset,        "AAC Data");
                     audio_stream_Count=false; //Unable to parse it
@@ -1200,6 +1371,7 @@ void File_Flv::audio_AAC()
                 break;
         case 1 :
                 //Parsing
+                Demux(Buffer+Buffer_Offset+(size_t)Element_Offset, (size_t)(Element_Size-Element_Offset), ContentType_MainStream);
                 Open_Buffer_Continue(Stream[Stream_Audio].Parser);
 
                 audio_stream_Count=false; //No need of more
