@@ -1,20 +1,9 @@
-// MediaInfoDLL - All info about media files, for DLL
-// Copyright (C) 2002-2009 Jerome Martinez, Zen@MediaArea.net
-//
-// This library is free software: you can redistribute it and/or modify it
-// under the terms of the GNU Lesser General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// any later version.
-//
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU Lesser General Public License for more details.
-//
-// You should have received a copy of the GNU Lesser General Public License
-// along with this library. If not, see <http://www.gnu.org/licenses/>.
-//
-//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+/*  Copyright (c) MediaArea.net SARL. All Rights Reserved.
+ *
+ *  Use of this source code is governed by a BSD-style license that can
+ *  be found in the License.html file in the root of the source tree.
+ */
+
 //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 //
 // Public DLL interface implementation
@@ -45,9 +34,10 @@
 #include "ZenLib/CriticalSection.h"
 #include <map>
 #include <vector>
-#include <iostream>
+#include <clocale>
 using namespace MediaInfoLib;
 using namespace ZenLib;
+using namespace std;
 //---------------------------------------------------------------------------
 
 //***************************************************************************
@@ -80,17 +70,17 @@ static bool utf8=false;
 const char* WC2MB(void* Handle, const wchar_t* Text)
 {
     CriticalSectionLocker Locker(Critical);
-    
+
     //Coherancy
     if (MI_Handle.find(Handle)==MI_Handle.end())
     {
         if (Handle==NULL && v07Mode)
-            MediaInfo_Info_Ansi="Note to developper : you must create an object before using this method";
+            MediaInfo_Info_Ansi="Note to developer : you must create an object before using this method";
         else
             MediaInfo_Info_Ansi="Your software uses an outdated interface, you must use MediaInfo.DLL 0.6 instead"; //Compatibility <0.7 : return a message
         return MediaInfo_Info_Ansi.c_str();
     }
-    
+
     //Adaptation
     if (utf8)
        MI_Handle[Handle]->Ansi=Ztring(Text).To_UTF8();
@@ -109,7 +99,7 @@ const wchar_t* MB2WC(void* Handle, const char* Text)
         MI_FromAnsi[Handle].Unicode.resize(2);
     }
     MI_FromAnsi[Handle].Pos=(MI_FromAnsi[Handle].Pos==0?1:0); //Changing the position
-       
+
     //Adaptation
     if (utf8)
         return MI_FromAnsi[Handle].Unicode[MI_FromAnsi[Handle].Pos].From_UTF8(Text).c_str();
@@ -175,7 +165,7 @@ const wchar_t* MB2WC(void* Handle, const char* Text)
             MediaInfo_Info_Unicode=L"Your software uses an outdated interface, You must use MediaInfoList.DLL 0.6 instead"; \
             return MediaInfo_Info_Unicode.c_str(); \
         } \
-        MI_Handle[NULL]->Unicode==L"Note to developper : you must create an object before"; \
+        MI_Handle[NULL]->Unicode==L"Note to developer : you must create an object before"; \
         return MI_Handle[NULL]->Unicode.c_str(); \
     } \
 
@@ -298,6 +288,11 @@ size_t          __stdcall MediaInfoA_Open_Buffer_Finalize (void* Handle)
     return MediaInfo_Open_Buffer_Finalize(Handle);
 }
 
+size_t          __stdcall MediaInfoA_Open_NextPacket (void* Handle)
+{
+    return MediaInfo_Open_NextPacket(Handle);
+}
+
 size_t          __stdcall MediaInfoA_Save (void* Handle)
 {
     return MediaInfo_Save(Handle);
@@ -334,6 +329,16 @@ size_t          __stdcall MediaInfoA_Set (void* Handle, const char* ToSet, Media
     return MediaInfo_Set(Handle, MB2WC(Handle, ToSet), StreamKind, StreamNumber, MB2WC(Handle, Parameter), MB2WC(Handle, OldParameter));
 }
 
+size_t          __stdcall MediaInfoA_Output_Buffer_Get (void* Handle, const char* Value)
+{
+    return MediaInfo_Output_Buffer_Get(Handle, MB2WC(Handle, Value));
+}
+
+size_t          __stdcall MediaInfoA_Output_Buffer_GetI (void* Handle, size_t Pos)
+{
+    return MediaInfo_Output_Buffer_GetI(Handle, Pos);
+}
+
 const char*     __stdcall MediaInfoA_Option (void* Handle, const char* Option, const char* Value)
 {
     return WC2MB(Handle, MediaInfo_Option(Handle, MB2WC(Handle, Option), MB2WC(Handle, Value)));
@@ -443,9 +448,6 @@ void*           __stdcall MediaInfo_New ()
     {
         MI_Handle[NULL]=new MI_List; //Generic Handle
         MI_Handle[NULL]->Ansi.clear();
-
-        //Localisation
-        setlocale(LC_ALL, """""");
     }
     v07Mode=true; //Application is compatible with v0.7+ interface
 
@@ -558,6 +560,13 @@ size_t          __stdcall MediaInfo_Open_Buffer_Finalize (void* Handle)
     EXECUTE_SIZE_T(MediaInfo, Open_Buffer_Finalize(), Debug+="MediaInfo_Open_Buffer_Finalize, will return ";Debug+=ZenLib::Ztring::ToZtring((size_t)ToReturn).To_Local().c_str();Debug+="\r\n";)
 }
 
+size_t          __stdcall MediaInfo_Open_NextPacket (void* Handle)
+{
+    INTEGRITY_SIZE_T(Debug+="MediaInfo_Open_NextPacket, Handle=";Debug+=ZenLib::Ztring::ToZtring((size_t)Handle).To_Local().c_str();Debug+=", ";Debug+="\r\n";)
+
+    EXECUTE_SIZE_T(MediaInfo, Open_NextPacket(), Debug+="MediaInfo_Open_NextPacket, will return ";Debug+=ZenLib::Ztring::ToZtring((size_t)ToReturn).To_Local().c_str();Debug+="\r\n";)
+}
+
 size_t          __stdcall MediaInfo_Save (void* Handle)
 {
     MANAGE_INT(MediaInfo, Save(), Debug+="Save, Handle=";Debug+=ZenLib::Ztring::ToZtring((size_t)Handle).To_Local().c_str();Debug+="\r\n";, Debug+="Save, will return ";Debug+=ZenLib::Ztring::ToZtring((size_t)ToReturn).To_Local().c_str();Debug+="\r\n";)
@@ -600,6 +609,16 @@ size_t          __stdcall MediaInfo_Set (void* Handle, const wchar_t* ToSet, Med
     MANAGE_INT(MediaInfo, Set(ToSet, (stream_t)StreamKind, StreamNumber, Parameter, OldParameter), Debug+="Set, Handle=";Debug+=ZenLib::Ztring::ToZtring((size_t)Handle).To_Local().c_str();Debug+="\r\n";, Debug+="Set, will return ";Debug+=ZenLib::Ztring::ToZtring((size_t)ToReturn).To_Local().c_str();Debug+="\r\n";)
 }
 
+size_t          __stdcall MediaInfo_Output_Buffer_Get (void* Handle, const wchar_t* Value)
+{
+    MANAGE_INT(MediaInfo, Output_Buffer_Get(Value), ;, ;)
+}
+
+size_t          __stdcall MediaInfo_Output_Buffer_GetI (void* Handle, size_t Pos)
+{
+    MANAGE_INT(MediaInfo, Output_Buffer_Get(Pos), ;, ;)
+}
+
 const wchar_t*     __stdcall MediaInfo_Option (void* Handle, const wchar_t* Option, const wchar_t* Value)
 {
     MEDIAINFO_DEBUG(Debug+="Option, Handle=";Debug+=ZenLib::Ztring::ToZtring((size_t)Handle).To_Local().c_str();Debug+=", Option=";Debug+=ZenLib::Ztring(Option).To_Local().c_str();Debug+="\r\n";)
@@ -610,9 +629,6 @@ const wchar_t*     __stdcall MediaInfo_Option (void* Handle, const wchar_t* Opti
     {
         MI_Handle[NULL]=new MI_List; //Generic Handle
         MI_Handle[NULL]->Ansi.clear();
-
-        //Localisation
-        setlocale(LC_ALL, """""");
     }
 
     //DLL only option
@@ -625,6 +641,11 @@ const wchar_t*     __stdcall MediaInfo_Option (void* Handle, const wchar_t* Opti
         MI_Handle[NULL]->Unicode=L"OK";
         return MI_Handle[NULL]->Unicode.c_str();
     }
+    if (ZenLib::Ztring(Option).Compare(L"setlocale_LC_CTYPE", L"=="))
+    {
+        setlocale(LC_CTYPE, utf8?ZenLib::Ztring(Value).To_UTF8().c_str():ZenLib::Ztring(Value).To_Local().c_str());
+        return MI_Handle[NULL]->Unicode.c_str();
+    }
 
     if (Handle)
     {
@@ -635,15 +656,15 @@ const wchar_t*     __stdcall MediaInfo_Option (void* Handle, const wchar_t* Opti
                 MediaInfo_Info_Unicode=L"Your software uses an outdated interface, You must use MediaInfoList.DLL 0.6 instead";
                 return MediaInfo_Info_Unicode.c_str();
             }
-              MI_Handle[NULL]->Unicode==L"Note to developper : you must create an object before";
+            MI_Handle[NULL]->Unicode==L"Note to developer : you must create an object before";
             return MI_Handle[NULL]->Unicode.c_str();
         }
         EXECUTE_STRING(MediaInfo, Option(Option, Value), Debug+="Option, will return ";Debug+=ToReturn.To_Local().c_str();Debug+="\r\n";)
     }
     else
     {
-        MI_Handle[Handle]->Unicode=MediaInfo::Option_Static(Option, Value);
-        return MI_Handle[Handle]->Unicode.c_str();
+        MI_Handle[NULL]->Unicode=MediaInfo::Option_Static(Option, Value);
+        return MI_Handle[NULL]->Unicode.c_str();
     }
 }
 
@@ -670,9 +691,6 @@ void*           __stdcall MediaInfoList_New ()
     {
         MI_Handle[NULL]=new MI_List; //Generic Handle
         MI_Handle[NULL]->Ansi.clear();
-
-        //Localisation
-        setlocale(LC_ALL, """""");
     }
     v07Mode=true; //Application is compatible with v0.7+ interface
 
@@ -808,9 +826,6 @@ const wchar_t*     __stdcall MediaInfoList_Option (void* Handle, const wchar_t*
     {
         MI_Handle[NULL]=new MI_List; //Generic Handle
         MI_Handle[NULL]->Ansi.clear();
-
-        //Localisation
-        setlocale(LC_ALL, """""");
     }
 
     //DLL only option
@@ -823,6 +838,11 @@ const wchar_t*     __stdcall MediaInfoList_Option (void* Handle, const wchar_t*
         MI_Handle[NULL]->Unicode=L"OK";
         return MI_Handle[NULL]->Unicode.c_str();
     }
+    if (ZenLib::Ztring(Option).Compare(L"setlocale_LC_CTYPE", L"=="))
+    {
+        setlocale(LC_CTYPE, utf8?ZenLib::Ztring(Value).To_UTF8().c_str():ZenLib::Ztring(Value).To_Local().c_str());
+        return MI_Handle[NULL]->Unicode.c_str();
+    }
 
     if (Handle)
     {
@@ -833,15 +853,15 @@ const wchar_t*     __stdcall MediaInfoList_Option (void* Handle, const wchar_t*
                 MediaInfo_Info_Unicode=L"Your software uses an outdated interface, You must use MediaInfoList.DLL 0.6 instead";
                 return MediaInfo_Info_Unicode.c_str();
             }
-            MI_Handle[NULL]->Unicode==L"Note to developper : you must create an object before";
+            MI_Handle[NULL]->Unicode==L"Note to developer : you must create an object before";
             return MI_Handle[NULL]->Unicode.c_str();
         }
         EXECUTE_STRING(MediaInfoList, Option(Option, Value), Debug+="Option(L), will return ";Debug+=ToReturn.To_Local().c_str();Debug+="\r\n";)
     }
     else
     {
-        MI_Handle[Handle]->Unicode=MediaInfoList::Option_Static(Option, Value);
-        return MI_Handle[Handle]->Unicode.c_str();
+        MI_Handle[NULL]->Unicode=MediaInfoList::Option_Static(Option, Value);
+        return MI_Handle[NULL]->Unicode.c_str();
     }
 }
 
